@@ -15,11 +15,9 @@
 # this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 #
-'''
-Facilities for handling the D-BUS driven configuration backend.
-'''
+'''Facilities for handling the D-BUS driven configuration backend.'''
 
-import dbus, dbus.service, gobject, shutil
+import dbus, dbus.service, gobject, logging, shutil
 import errno, os, os.path, re, pty, signal, tempfile
 
 from gettext               import gettext as _
@@ -30,32 +28,30 @@ from StringIO              import StringIO
 # for old flavours the internal _dbus_bindings module must be used.
 
 try:
+    # pylint: disable-msg=E0611
     from dbus.lowlevel import HANDLER_RESULT_NOT_YET_HANDLED
+
 except ImportError:
     from _dbus_bindings import HANDLER_RESULT_NOT_YET_HANDLED
 
 class AccessDeniedException(dbus.DBusException):
-    '''
-    This exception is raised when some operation is not permitted.
-    '''
+    '''This exception is raised when some operation is not permitted.'''
 
     _dbus_error_name = 'org.gnome.LircProperties.AccessDeniedException'
 
 class UnsupportedException(dbus.DBusException):
-    '''
-    This exception is raised when some operation is not supported.
-    '''
+    '''This exception is raised when some operation is not supported.'''
 
     _dbus_error_name = 'org.gnome.LircProperties.UnsupportedException'
 
 class UsageError(dbus.DBusException):
-    '''
-    This exception is raised when some operation was not used properly.
-    '''
+    '''This exception is raised when some operation was not used properly.'''
 
     _dbus_error_name = 'org.gnome.LircProperties.UsageError'
 
 class PolicyKitService(dbus.service.Object):
+    '''A D-BUS service that uses PolicyKit for authorization.'''
+
     def _check_permission(self, sender, action=config.POLICY_KIT_ACTION):
         '''
         Verifies if the specified action is permitted, and raises
@@ -67,6 +63,7 @@ class PolicyKitService(dbus.service.Object):
         try:
             if sender:
                 kit = dbus.SystemBus().get_object('org.freedesktop.PolicyKit', '/')
+                kit = dbus.Interface(kit, 'org.freedesktop.PolicyKit')
                 pid = dbus.UInt32(os.getpid())
 
                 if 'yes' != kit.IsProcessAuthorized(action, pid, False):
@@ -81,9 +78,25 @@ class PolicyKitService(dbus.service.Object):
             raise AccessDeniedException(ex.message)
 
 class ExternalToolDriver(PolicyKitService):
+    '''
+    A D-BUS service which mainly is implemented
+    as wrapper arround some external program.
+    '''
+
+    # pylint: disable-msg=C0103,E0602
+
     INTERFACE_NAME = 'org.gnome.LircProperties.ExternalToolDriver'
 
-    def __spawn_external_tool(self):
+    def __init__(self, connection, path='/'):
+        super(ExternalToolDriver, self).__init__(connection, path)
+
+        self.__line_buffer = ''
+        self.__pid = -1
+        self.__fd = -1
+
+    def _spawn_external_tool(self):
+        '''Launches the external tool backing this service.'''
+
         pid, fd = pty.fork()
 
         if 0 == pid:
@@ -91,52 +104,67 @@ class ExternalToolDriver(PolicyKitService):
             assert False # should not be reached
 
         os.waitpid(pid, os.P_NOWAIT)
-
         return pid, fd
 
     def __io_handler(self, fd, condition):
+        '''Handles I/O events related to the backing external tool.'''
+
         if condition & gobject.IO_IN:
+            # Read next chunk for the file descriptor and buffer it:
             chunk = os.read(fd, 4096)
             self._on_next_chunk(chunk)
-
             self.__line_buffer += chunk
 
+            # Extract complete lines from buffer:
             while True:
                 linebreak = self.__line_buffer.find('\n')
 
-                if linebreak < 0: break
+                # Abort, since the next line isn't complete yet:
+                if linebreak < 0:
+                    break
 
+                # Extract next line and normalize it:
                 line = self.__line_buffer[:linebreak].rstrip()
                 self.__line_buffer = self.__line_buffer[linebreak + 1:]
 
+                # Handle next line:
                 self._on_next_line(line)
 
         if condition & gobject.IO_HUP:
+            # Shutdown the service, when the backing tool terminates:
             self._on_hangup()
             return False
 
+        # Keep the handler alive:
         return True
 
     def _send_response(self, response='\n'):
+        '''Sends a response, by default just a line feed, to the external tool.'''
+
         os.write(self.__fd, response)
 
     @dbus.service.method(dbus_interface=INTERFACE_NAME,
                          in_signature='', out_signature='',
                          sender_keyword='sender')
     def Execute(self, sender=None):
+        '''Requests the service to launch the backing tool.'''
+
         self._check_permission(sender)
 
         self.__line_buffer = ''
-        self.__pid, self.__fd = self.__spawn_external_tool()
+        self.__pid, self.__fd = self._spawn_external_tool()
 
-        gobject.io_add_watch(self.__fd,
-                             gobject.IO_IN | gobject.IO_HUP,
-                             self.__io_handler)
+        if -1 != self.__fd:
+            gobject.io_add_watch(self.__fd,
+                                 gobject.IO_IN | gobject.IO_HUP,
+                                 self.__io_handler)
 
     @dbus.service.method(dbus_interface=INTERFACE_NAME,
                          in_signature='', out_signature='',
                          sender_keyword='sender')
     def Proceed(self, sender=None):
+        '''Requests the service to send a line feed to the backing tool.'''
+
         self._check_permission(sender)
         self._send_response('\n')
 
@@ -144,6 +172,8 @@ class ExternalToolDriver(PolicyKitService):
                          in_signature='', out_signature='',
                          sender_keyword='sender')
     def Release(self, sender=None):
+        '''Releases the service and shuts down the backing tool, when still alive.'''
+
         self._check_permission(sender)
 
         try:
@@ -159,29 +189,45 @@ class ExternalToolDriver(PolicyKitService):
         self.remove_from_connection()
 
     @dbus.service.signal(dbus_interface=INTERFACE_NAME, signature='')
-    def ReportProgress(self): pass
+    def ReportProgress(self):
+        '''Signals that the backing tool reported some progress.'''
+
     @dbus.service.signal(dbus_interface=INTERFACE_NAME, signature='s')
-    def ReportSuccess(self, message): pass
+    def ReportSuccess(self, message):
+        '''Signals that the backing tool reported sucess.'''
+
     @dbus.service.signal(dbus_interface=INTERFACE_NAME, signature='s')
-    def ReportFailure(self, message): pass
+    def ReportFailure(self, message):
+        '''Signals that the backing tool reported failure.'''
+
     @dbus.service.signal(dbus_interface=INTERFACE_NAME, signature='ss')
-    def RequestAction(self, title, details): pass
+    def RequestAction(self, title, details):
+        '''Signals that the backing tool requests some action.'''
 
-    def _on_run_external_tool(self): pass
-    def _on_next_chunk(self, line): pass
-    def _on_next_line(self, line): pass
-    def _on_hangup(self): pass
+    def _on_run_external_tool(self):
+        '''Executes the external tool.'''
+    def _on_next_chunk(self, chunk):
+        '''Processes the next chunk of output from the external tool.'''
+    def _on_next_line(self, line):
+        '''Processes the next line of output from the external tool.'''
+    def _on_hangup(self):
+        '''Handles termination of the external tool.'''
 
+    # pylint: disable-msg=W0212
     _pid = property(lambda self: self.__pid)
 
 class IrRecordDriver(ExternalToolDriver):
+    '''D-BUS service that runs irrecord.'''
+
+    # pylint: disable-msg=C0103
+
     # Following strings are some known error messages of irrecord 0.5,
     # as shipped with Ubuntu 7.10 on 2008-02-13:
 
     _errors = {
         'could not init hardware': _('Could not initialize hardware.'),
-        'gap not found, can\'t continue': _('No key presses recognized, gap not found.'),
-        'no data for 10 secs, aborting': _('No key presses recognized, aborting.'),
+        'gap not found, can\'t continue': _('No key presses recognized. Gap not found.'),
+        'no data for 10 secs, aborting': _('No key presses recognized. Aborting.'),
     }
 
     # Following strings indicate state changes in irrecord 0.5,
@@ -197,8 +243,9 @@ class IrRecordDriver(ExternalToolDriver):
     # Last instance index for automatic object path creation:
     __last_instance = 0
 
+    # pylint: disable-msg=R0913
     def __init__(self, connection, driver, device,
-                 filename='lircd.conf', path=None):
+                 filename='lircd.conf.learning', path=None):
         assert not os.path.isabs(filename)
 
         if not path:
@@ -217,6 +264,8 @@ class IrRecordDriver(ExternalToolDriver):
         super(IrRecordDriver, self).__init__(connection, path)
 
     def _on_run_external_tool(self):
+        '''Enters to the working directory and executes irrecord.'''
+
         os.chdir(self._workdir)
         self._prepare_workdir()
 
@@ -227,20 +276,26 @@ class IrRecordDriver(ExternalToolDriver):
         os.execve(args[0], filter(None, args), env)
 
     def _prepare_workdir(self):
-        pass
+        '''Virtual method for preparation of the working directory.'''
 
     def _cleanup_workdir(self):
+        '''Virtual method for cleaning up the working directory.'''
+
         print 'cleaning up %s...' % self._workdir
 
         for name in os.listdir(self._workdir):
             os.unlink(os.path.join(self._workdir, name))
 
     def _on_hangup(self):
+        '''Shutdown the driver, when irrecord terminates unexpectedly.'''
+
         if list(self.locations):
-            self.ReportFailure(_('Custom remote configuration aborted unexpectedly.'))
+            self.ReportFailure(_('Custom remote control configuration aborted unexpectedly.'))
             self.Release()
 
     def _find_error_messages(self, line):
+        '''Tries to identify error messages in line and reports them.'''
+
         for token, message in self._errors.items():
             if line.find(token) >= 0:
                 self.ReportFailure(message)
@@ -254,15 +309,23 @@ class IrRecordDriver(ExternalToolDriver):
         os.rmdir(self._workdir)
 
 class DetectParametersDriver(IrRecordDriver):
+    '''D-BUS service that runs irrecord for detecting basic remote properties.'''
+
     def __init__(self, connection, driver, device):
         super(DetectParametersDriver, self).__init__(connection, driver, device)
         self.__report_progress = False
 
     def _prepare_workdir(self):
+        '''Removes the configuration file from working directory when needed.'''
+
         if os.path.exists(self._filename):
             os.unlink(self._filename)
 
     def _on_next_line(self, line):
+        '''Processes the next line of irrecord output.'''
+
+        # pylint: disable-msg=R0911
+
         print '%d:%s' % (self._pid, line)
 
         # Identify known error messages:
@@ -277,7 +340,7 @@ class DetectParametersDriver(IrRecordDriver):
             return
 
         if line.startswith(self._token_hold_one_button):
-            self.RequestAction(_('Hold down an arbitrary button.'), '')
+            self.RequestAction(_('Hold down any remote control button.'), '')
             return
 
         if line.startswith(self._token_random_buttons):
@@ -299,8 +362,8 @@ class DetectParametersDriver(IrRecordDriver):
             self.RequestAction(
                 _('Press a button repeatedly as fast as possible.'),
                 _('Make sure you keep pressing the <b>same</b> button and that you ' +
-                  '<b>do not hold</b> the button down!\nWait a bit between button ' +
-                  'presses, if you can not see any progress.'))
+                  '<b>do not hold</b> the button down.\nWait a little between button ' +
+                  'presses if you cannot see any progress.'))
 
             return
 
@@ -312,18 +375,30 @@ class DetectParametersDriver(IrRecordDriver):
             return
 
     def _on_next_chunk(self, chunk):
+        '''
+        Identifies progress indications in the next chunk of irrecord output,
+        and reports them.
+        '''
+
         if '.' == chunk:
             self.ReportProgress()
 
 class LearnKeyCodeDriver(IrRecordDriver):
+    '''D-BUS service that runs irrecord for detecting key-codes.'''
+
+    # pylint: disable-msg=R0913
     def __init__(self, connection, driver, device, configuration, keys):
         super(LearnKeyCodeDriver, self).__init__(connection, driver, device)
         self.__keys, self.__configuration = keys, configuration
 
     def _prepare_workdir(self):
+        '''Writes the supplied configuration file into the  working directory.'''
+
         open(self._filename, 'w').write(self.__configuration)
 
     def _on_next_line(self, line):
+        '''Processes the next line of irrecord output.'''
+
         print '%d:%s' % (self._pid, line)
 
         # Identify known error messages:
@@ -348,13 +423,18 @@ class LearnKeyCodeDriver(IrRecordDriver):
         if line.startswith(self._token_finished):
             configuration = self._find_configuration()
 
-            if configuration: self.ReportSuccess(configuration)
-            else: self.ReportFailure(_('Cannot find recorded key codes'))
+            if configuration:
+                self.ReportSuccess(configuration)
+
+            else:
+                self.ReportFailure(_('Cannot find recorded key codes'))
 
             self.Release()
             return
 
     def _find_configuration(self):
+        '''Finds the configuration file generated by irrecord.'''
+
         for suffix in '.new', '.conf', '':
             filename = os.path.join(self._workdir, self._filename + suffix)
 
@@ -363,26 +443,27 @@ class LearnKeyCodeDriver(IrRecordDriver):
 
         return None
 
-    def _on_run_external_tool(self):
+    def _spawn_external_tool(self):
+        '''Runs irrecord with --resume switch when supported.'''
         irrecord = os.popen('%s --help' % config.LIRC_IRRECORD)
 
         if -1 == irrecord.read().find('--resume'):
-            # TODO: Should be an UnsupportedException in LearnKeyCode(),
-            # but when raising that exception from there, the execeptions
-            # message property also contains a stack trace.
+            logging.warning('irrecord doesn\'t have --resume switch')
+
             self.ReportFailure(_(
-                'Installed lirc package doesn\'t support self-contained ' +
-                'key code yet. Update to latest lirc package if possible.'))
+                'The installed lirc does not support key code learning ' +
+                'because its irrecord command does not have the --resume ' +
+                'option. Please file a bug against your Linux distribution.'))
+
+            return 0, -1
 
         self._cmdargs.insert(1, '--resume')
-
-        super(LearnKeyCodeDriver, self)._on_run_external_tool()
-
+        return super(LearnKeyCodeDriver, self)._spawn_external_tool()
 
 class BackendService(PolicyKitService):
-    '''
-    A D-Bus service that PolicyKit controls access to.
-    '''
+    '''A D-Bus service that PolicyKit controls access to.'''
+
+    # pylint: disable-msg=C0103,E0602
 
     INTERFACE_NAME = 'org.gnome.LircProperties.Mechanism'
     SERVICE_NAME   = 'org.gnome.LircProperties.Mechanism'
@@ -400,19 +481,32 @@ class BackendService(PolicyKitService):
                                        r'LIRCD_ARGS|LIRCD_CONF|VENDOR|MODEL)=')
     __re_start_lircd      = re.compile(r'^\s*START_LIRCD=')
 
-    def __init__(self, connection, path='/'):
+    def __init__(self, connection=None, path='/'):
+        if connection is None:
+            connection = get_service_bus()
+
         super(BackendService, self).__init__(connection, path)
 
+        self.__name = dbus.service.BusName(self.SERVICE_NAME, connection)
         self.__loop = gobject.MainLoop()
         self.__timeout = 0
 
         connection.add_message_filter(self.__message_filter)
 
     def __message_filter(self, connection, message):
-        if self.__timeout: self.__start_idle_timeout()
+        '''
+        D-BUS message filter that keeps the service alive,
+        as long as it receives message.
+        '''
+
+        if self.__timeout:
+            self.__start_idle_timeout()
+
         return HANDLER_RESULT_NOT_YET_HANDLED
 
     def __start_idle_timeout(self):
+        '''Restarts the timeout for terminating the service when idle.'''
+
         if self.__timeout:
             gobject.source_remove(self.__timeout)
 
@@ -420,6 +514,8 @@ class BackendService(PolicyKitService):
                                              self.__timeout_cb)
 
     def __timeout_cb(self):
+        '''Timeout callback that terminates the service when idle.'''
+
         # Keep service alive, as long as additional objects are exported:
         if self.connection.list_exported_child_objects('/'):
             return True
@@ -430,9 +526,7 @@ class BackendService(PolicyKitService):
         return False
 
     def run(self):
-        '''
-        Creates a GLib main loop for keeping the service alive.
-        '''
+        '''Creates a GLib main loop for keeping the service alive.'''
 
         print 'Running %s.' % self.SERVICE_NAME
         print 'Terminating it after %d seconds of inactivity.' % self.IDLE_TIMEOUT
@@ -440,11 +534,16 @@ class BackendService(PolicyKitService):
         self.__start_idle_timeout()
         self.__loop.run()
 
-    def _write_hardware_configuration(self, remote_values=None, receiver_values=None):
-        if not remote_values: remote_values = dict()
-        if not receiver_values: receiver_values = dict()
+    def _write_hardware_configuration(self,
+                                      remote_values=dict(),
+                                      receiver_values=dict(),
+                                      start_lircd=None):
+        '''Updates lirc's hardware.conf file on Debian/Ubuntu.'''
 
-        oldfile = os.path.join(config.LIRC_CONFDIR, 'hardware.conf')
+        if start_lircd is not None:
+            start_lircd = str(bool(start_lircd)).lower()
+
+        oldfile = config.LIRC_HARDWARE_CONF
         newfile = '%s.tmp' % oldfile
 
         if not os.path.isfile(oldfile):
@@ -464,9 +563,8 @@ class BackendService(PolicyKitService):
                 # Remove entry from the dict, so we know what was written.
                 value = remote_values.pop(match.group(1), None)
 
-                if None != value:
-                    print match.group(0), value
-                    print >>output, ('%s"%s"' % (match.group(0), value))
+                if value is not None:
+                    print >> output, ('%s"%s"' % (match.group(0), value))
                     continue
 
             # Deal with lines starting with RECEIVER_,
@@ -478,17 +576,19 @@ class BackendService(PolicyKitService):
                 # Remove entry from the dict, so we know what was written.
                 value = receiver_values.pop(match.group(1), None)
 
-                if None != value:
-                    print >>output, ('%s"%s"' % (match.group(0), value))
+                if value is not None:
+                    print >> output, ('%s"%s"' % (match.group(0), value))
                     continue
 
             # Deal with the START_LIRCD line:
-            # Unconditionally set it to "true" since we rely on lircd starting up.
 
             match = self.__re_start_lircd.match(line)
 
             if match:
-                print >>output, ('%strue' % match.group(0))
+                #TODO: pychecker says "gnome_lirc_properties/backend.py:588: Using a conditional statement with a constant value (true)"
+                value = (start_lircd is None and 'true') or start_lircd
+                print >> output, (match.group(0) + value)
+                start_lircd = None
                 continue
 
             output.write(line)
@@ -497,20 +597,25 @@ class BackendService(PolicyKitService):
         # and therefore just replaced:
 
         if remote_values:
-            print >>output, '\n# Remote settings required by gnome-lirc-properties'
+            print >> output, '\n# Remote settings required by gnome-lirc-properties'
         for key, value in remote_values.items():
-            print >>output, ('REMOTE_%s="%s"' % (key, value))
+            print >> output, ('REMOTE_%s="%s"' % (key, value))
 
         if receiver_values:
-            print >>output, '\n# Receiver settings required by gnome-lirc-properties'
+            print >> output, '\n# Receiver settings required by gnome-lirc-properties'
         for key, value in receiver_values.items():
-            print >>output, ('RECEIVER_%s="%s"' % (key, value))
+            print >> output, ('RECEIVER_%s="%s"' % (key, value))
+
+        if start_lircd is not None:
+            print >> output, '\n# Daemon settings required by gnome-lirc-properties'
+            print >> output, ('START_LIRCD=%s' % start_lircd)
 
         # Replace old file with new contents:
 
         os.unlink(oldfile)
         os.rename(newfile, oldfile)
 
+    # pylint: disable-msg=R0913
     @dbus.service.method(dbus_interface=INTERFACE_NAME,
                          in_signature='sssss', out_signature='',
                          sender_keyword='sender')
@@ -518,7 +623,8 @@ class BackendService(PolicyKitService):
                                    driver, device, modules,
                                    sender=None):
         '''
-        Update the /etc/lirc/hardware.conf file, so that lircd is started as specified.
+        Update the /etc/lirc/hardware.conf file,
+        so that lircd is started as specified.
         '''
 
         self._check_permission(sender)
@@ -571,46 +677,108 @@ class BackendService(PolicyKitService):
             self._write_hardware_configuration(remote_values=values)
 
         # Write remote configuration:
+        filename, include_needed = lirc.find_remote_config()
 
-        filename = os.path.join(config.LIRC_CONFDIR, 'lircd.conf')
+        if include_needed:
+            self.__write_include_statement(filename)
+
+        print contents
 
         print 'Updating %s...' % filename
         file(filename, 'w').write(contents)
 
+    @staticmethod
+    def __write_include_statement(redirect):
+        '''Write include statement to central lircd.conf file.'''
+
+        # read central lirc configuration file file:
+        lircd_config = os.path.join(config.LIRC_CONFDIR, 'lircd.conf')
+
+        try:
+            contents = open(lircd_config).read()
+
+        except IOError:
+            contents = ''
+
+        # drop entire configuration file, if it still contains embedded
+        # configuration files - for instance from Gutsy:
+        pattern = r'^\s*begin\s+remote\s*$'
+        pattern = re.compile(pattern, re.MULTILINE)
+        match   = pattern.search(contents)
+
+        if match:
+            contents = ''
+
+        # find existing include statement:
+        include_statement = 'include %s\n' % redirect
+        pattern = r'^\s*(#.*)?include\s+%s\s*$' % re.escape(redirect)
+        pattern = re.compile(pattern, re.MULTILINE)
+        match   = pattern.search(contents)
+
+        if match is None:
+            # no statement found, create entirely new file:
+
+            if not contents:
+                contents  = '# This configuration has been automatically generated\n'
+                contents += '# by the GNOME LIRC Properties control panel.\n'
+                contents += '#\n'
+                contents += '# Feel free to add any custom remotes to the configuration\n'
+                contents += '# via additional include directives or below the existing\n'
+                contents += '# Ubuntu include directives from your selected remote and/or\n'
+                contents += '# transmitter.\n'
+                contents += '#\n'
+
+            contents += '\n'
+            contents += '# Configuration selected with GNOME LIRC Properties\n'
+            contents += include_statement
+
+        elif match.group(1):
+            head = contents[:match.start()]
+            tail = contents[match.end():]
+            contents = (head + include_statement + tail)
+
+        else:
+            contents = None
+
+        if contents:
+            open(lircd_config, 'w').write(contents)
+
     @dbus.service.method(dbus_interface=INTERFACE_NAME,
                          in_signature='', out_signature='',
                          sender_keyword='sender')
     def ManageLircDaemon(self, action, sender=None):
-        '''
-        Starts the LIRC daemon.
-        '''
+        '''Starts the LIRC daemon.'''
 
         self._check_permission(sender)
 
         print 'Managing lircd: %s...' % action
-        args = '/etc/init.d/lirc', action
 
-        os.spawnv(os.P_WAIT, args[0], args)
+        if 'enable' == action:
+            self._write_hardware_configuration(start_lircd=True)
+
+        elif 'disable' == action:
+            self._write_hardware_configuration(start_lircd=False)
+
+        else:
+            args = '/etc/init.d/lirc', action
+            os.spawnv(os.P_WAIT, args[0], args)
 
     @dbus.service.method(dbus_interface=INTERFACE_NAME,
                          in_signature='ss', out_signature='o',
                          sender_keyword='sender')
     def DetectParameters(self, driver, device, sender=None):
-        '''
-        Detects parameters of the IR remote by running irrecord.
-        '''
+        '''Detects parameters of the IR remote by running irrecord.'''
 
         self._check_permission(sender)
 
         return DetectParametersDriver(self.connection, driver, device)
 
+    # pylint: disable-msg=R0913
     @dbus.service.method(dbus_interface=INTERFACE_NAME,
                          in_signature='sssas', out_signature='o',
                          sender_keyword='sender')
     def LearnKeyCode(self, driver, device, configuration, keys, sender=None):
-        '''
-        Learn the scan code of some IR remote key by running irrecord.
-        '''
+        '''Learn the scan code of some IR remote key by running irrecord.'''
 
         self._check_permission(sender)
 
@@ -623,34 +791,37 @@ class BackendService(PolicyKitService):
                          in_signature='s', out_signature='',
                          sender_keyword='sender')
     def InstallRemoteDatabase(self, filename, sender=None):
-        '''
-        Update the customized receiver database.
-        '''
+        '''Update the customized receiver database.'''
 
         # Copy the tarball with updates into our data folder
         # to allow atomic replacement of the old tarball:
         tarball = os.path.join(config.PACKAGE_DIR, 'remotes-update.tar.gz')
+
+        if not os.path.isdir(config.PACKAGE_DIR):
+            os.makedirs(config.PACKAGE_DIR)
+
         shutil.copyfile(filename, tarball)
 
         # Now practice the atomic replacement:
         os.rename(tarball, config.LIRC_REMOTES_TARBALL)
 
+        # Finally adjust timestamps of the archive:
+        timestamps = os.path.getatime(filename), os.path.getmtime(filename)
+        os.utime(config.LIRC_REMOTES_TARBALL, timestamps)
+
 def get_service_bus():
-    '''
-    Retrieves a reference to the D-BUS system bus.
-    '''
+    '''Retrieves a reference to the D-BUS system bus.'''
 
     return dbus.SystemBus()
 
 def get_service(bus=None):
-    '''
-    Retrieves a reference to the D-BUS driven configuration service.
-    '''
+    '''Retrieves a reference to the D-BUS driven configuration service.'''
 
     if not bus:
         bus = get_service_bus()
 
     service = bus.get_object(BackendService.SERVICE_NAME, '/')
+    service = dbus.Interface(service, BackendService.INTERFACE_NAME)
 
     return service
 
@@ -660,12 +831,7 @@ if __name__ == '__main__':
     from dbus.mainloop.glib import DBusGMainLoop
     DBusGMainLoop(set_as_default=True)
 
-    # Get system bus and acquire service name.
-
-    bus = get_service_bus()
-    name = dbus.service.BusName(BackendService.SERVICE_NAME, bus)
-
     # Run the service.
 
-    BackendService(bus).run()
+    BackendService().run()
 
