@@ -23,14 +23,12 @@ import dbus, logging, gobject, gtk
 
 from gettext                         import gettext as _
 from gnome_lirc_properties           import backend, config, lirc, model, net
-from gnome_lirc_properties.ui.common import show_message
+from gnome_lirc_properties.ui.common import show_message, thread_callback
 from StringIO                        import StringIO
 from time                            import time
 
 class CustomConfiguration(object):
-    '''
-    This window allows the user to detect each key individually.
-    '''
+    '''This window allows the user to detect each key individually.'''
 
     default_keys = model.DictionaryStore(
         values=dict([(c.key, c.display_name) for c in
@@ -40,8 +38,11 @@ class CustomConfiguration(object):
     def __init__(self, glade_xml):
         self.__remote = None
 
-        # Remember row of column that currently learned:
+        # irrecord drivers:
+        self.__detection_driver = None
         self.__learning_driver = None
+
+        # Remember row of column that currently learned:
         self.__learning_timeout = 0
         self.__learning_row = None
 
@@ -50,9 +51,7 @@ class CustomConfiguration(object):
 
     # pylint: disable-msg=W0201
     def __setup_ui(self, glade_xml):
-        '''
-        Initialize widgets.
-        '''
+        '''Initialize widgets.'''
 
         self.__ui = glade_xml
         self.__ui.signal_autoconnect(self)
@@ -65,8 +64,10 @@ class CustomConfiguration(object):
         self.__button_ok            = self.__ui.get_widget('button_ok')
         self.__button_upload        = self.__ui.get_widget('button_upload')
         self.__button_detect_basics = self.__ui.get_widget('button_detect_basics')
+        self.__button_keys_learn    = self.__ui.get_widget('button_keys_learn')
         self.__button_keys_remove   = self.__ui.get_widget('button_keys_remove')
         self.__button_keys_clear    = self.__ui.get_widget('button_keys_clear')
+        self.__button_keys_add      = self.__ui.get_widget('button_keys_add')
 
         # setup model page:
         self.__entry_vendor      = self.__ui.get_widget('entry_vendor')
@@ -85,11 +86,14 @@ class CustomConfiguration(object):
         # apply "secondary" child property: for some reason libglade ignores it
         self.__dialog.action_area.set_child_secondary(self.__button_upload, True)
 
+        # apply stock icon to learn button
+        image = gtk.image_new_from_stock(gtk.STOCK_MEDIA_RECORD,
+                                         gtk.ICON_SIZE_BUTTON)
+        self.__button_keys_learn.set_image(image)
+
     # pylint: disable-msg=W0201
     def __setup_basics(self):
-        '''
-        Initialize widgets of the "Basics" page.
-        '''
+        '''Initialize widgets of the "Basics" page.'''
 
         self.__page_basics               = self.__ui.get_widget('page_basics')
         self.__treeview_basics           = self.__ui.get_widget('treeview_basics')
@@ -108,9 +112,7 @@ class CustomConfiguration(object):
 
     # pylint: disable-msg=W0201
     def __setup_keys(self):
-        '''
-        Initialize widgets of the "Keys" page.
-        '''
+        '''Initialize widgets of the "Keys" page.'''
 
         self.__page_keys       = self.__ui.get_widget('page_keys')
         self.__treeview_keys   = self.__ui.get_widget('treeview_keys')
@@ -131,9 +133,7 @@ class CustomConfiguration(object):
 
         # pylint: disable-msg=W0613
         def edited(cell, path, new_name):
-            '''
-            Update the keys model, when editing of the key-name column has ended.
-            '''
+            '''Update the keys model, when editing of the key-name column has ended.'''
 
             keys_model = self.__treeview_keys.get_model()
             keys_iter = keys_model.get_iter(path)
@@ -176,9 +176,7 @@ class CustomConfiguration(object):
         selection.set_mode(gtk.SELECTION_BROWSE)
 
     def __update_basics_model(self, properties):
-        '''
-        Update the entries stored in the basic properties model.
-        '''
+        '''Update the entries stored in the basic properties model.'''
 
         if not properties:
             properties = dict()
@@ -253,9 +251,7 @@ class CustomConfiguration(object):
         return True
 
     def __start_learning(self, path):
-        '''
-        Start learning of a remote control's key code.
-        '''
+        '''Start learning of a remote control's key code.'''
 
         keys = self.__treeview_keys.get_model()
 
@@ -272,10 +268,9 @@ class CustomConfiguration(object):
         device = self.__device or ''
 
         # pylint: disable-msg=W0613
+        @thread_callback
         def report_success(result, sender=None):
-            '''
-            Handle success reported by the service backend.
-            '''
+            '''Handle success reported by the service backend.'''
 
             self.__learning_driver = None
             self.__stop_learning()
@@ -288,10 +283,9 @@ class CustomConfiguration(object):
                 self.__treeview_keys.get_model().update(remote.key_codes)
 
         # pylint: disable-msg=W0613
+        @thread_callback
         def report_failure(message, sender=None):
-            '''
-            Handle failures reported by the service backend.
-            '''
+            '''Handle failures reported by the service backend.'''
 
             show_message(self.__dialog, _('Learning of Key Code Failed'), message)
 
@@ -302,7 +296,9 @@ class CustomConfiguration(object):
             bus     = backend.get_service_bus()
             service = backend.get_service(bus)
             driver  = service.LearnKeyCode(driver, device, configuration, [mapping.key])
-            driver  = bus.get_object(service.requested_bus_name, driver)
+
+            driver = bus.get_object(service.requested_bus_name, driver)
+            driver = dbus.Interface(driver, backend.ExternalToolDriver.INTERFACE_NAME)
 
             self.__learning_driver = driver
 
@@ -312,12 +308,12 @@ class CustomConfiguration(object):
             driver.Execute()
 
         except dbus.DBusException, ex:
-            report_failure(ex.message)
+            report_failure.__direct__(ex.message)
+
+        self._on_dialog_changed()
 
     def __stop_learning(self):
-        '''
-        Stop learning of a remote control's key code.
-        '''
+        '''Stop learning of a remote control's key code.'''
 
         if self.__learning_driver:
             try:
@@ -333,50 +329,77 @@ class CustomConfiguration(object):
             keys.set_state(self.__learning_row.get_path(), None)
             self.__learning_row = None
 
+        self._on_dialog_changed()
+
     # pylint: disable-msg=C0103,W0613
     def _on_treeview_keys_selection_changed(self, selection):
-        '''
-        Handle selection of row in the "keys" tree view.
-        '''
+        '''Handle selection of row in the "keys" tree view.'''
+        self._on_dialog_changed()
         self.__stop_learning()
 
     # pylint: disable-msg=W0613
     def _on_treeview_keys_row_activated(self, view, path, column):
-        '''
-        Handle activation of row in the "keys" tree view.
-        '''
+        '''Handle activation of row in the "keys" tree view.'''
         self.__start_learning(path)
 
-    def _on_upload_button_clicked(self, button):
-        '''
-        Handle the "Upload" button's "clicked" signal.
-        '''
+    def _on_button_upload_clicked(self, button):
+        '''Handle the "Upload" button's "clicked" signal.'''
 
-        # A site with a known working file upload feature:
-        base_uri = ('http://www.glom.org/wiki/index.php?title=')
-        login_uri = (base_uri + 'Special:Userlogin&action=submitlogin&' +
-                                'type=login&wpLoginattempt=Log+in')
-        upload_uri = (base_uri + 'Special:Upload&wpUpload=Upload+file')
+        def configuration_problem(message):
+            '''
+            Informs the user about configuration file problems,
+            and ask the user for accption or rejection.
+            '''
 
-        # TODO: Use the URIs provided to configure:
-        # login_uri = config.URI_UPLOADS_LOGIN
-        # upload_uri = config.URI_UPLOADS
+            response_buttons = (
+                (gtk.RESPONSE_ACCEPT, _('_Upload Configuration')),
+                (gtk.RESPONSE_REJECT, gtk.STOCK_CANCEL),
+            )
 
-        uploader = net.HttpUploader(upload_uri, login_uri,
-                                    'Murrayc', 'enterpasswordhere')
+            message = '%s %s' % (message, _('Do you really want to upload this configuration?'))
 
-        # TODO: For mediawiki, the name should always
-        # start with a capital letter, and have no spaces.
-        lircd_conf_name = (
-            '%s_MODEL_%s_MANUFACTURER_%s' % ('Test_lircd.conf',
-            self.product_name, self.vendor_name))
+            return (gtk.RESPONSE_ACCEPT != show_message(
+                    self.__dialog, _('Configuration Problems'),
+                    message, buttons=response_buttons))
 
-        uploader.post_file(config.LIRC_CONF_FILEPATH, lircd_conf_name)
+        def on_upload_finished(message):
+            '''Informs the user about succeeding uploads.'''
+
+            show_message(self.__dialog, _('Upload Succeeded'),
+                         message, message_type=gtk.MESSAGE_INFO)
+            self.__dialog.set_sensitive(True)
+
+        def on_upload_failure(message):
+            '''Informs the user about upload failures.'''
+
+            show_message(self.__dialog, _('Upload Failed'), message)
+            self.__dialog.set_sensitive(True)
+
+        keys_model = self.__treeview_keys.get_model()
+
+        if 0 == keys_model.count_keys(lirc.KeyCodeCategory.DEFAULT):
+            if configuration_problem(_('This configuration has no keys for the default ' +
+                                       'namespace. Most applications won\'t be able to ' +
+                                       'use this configuration.')):
+                return
+
+        elif 0 != keys_model.count_keys(lirc.KeyCodeCategory.CUSTOM):
+            if configuration_problem(_('Some keys in this configuration have names ' +
+                                       'which do not belong to any standardized namespace.' +
+                                       'Most applications won\'t be able to use those keys.')):
+                return
+
+        self.__dialog.set_sensitive(False)
+
+        net.post_file(target_uri=config.URI_UPLOADS,
+                      filename=lirc.find_remote_config(),
+                      content=self.__remote.configuration,
+                      context=_('customized configuration file'),
+                      finished_callback=on_upload_finished,
+                      failure_callback=on_upload_failure)
 
     def __retreive_remote_name(self):
-        '''
-        Build the symbolic name of the currently edited remote.
-        '''
+        '''Build the symbolic name of the currently edited remote.'''
 
         name = self.__remote and self.__remote.name
 
@@ -387,16 +410,11 @@ class CustomConfiguration(object):
         return name
 
     def _on_detect_button_clicked(self, button):
-        '''
-        Handle the "Detect" button's "clicked" signal.
-        '''
-
-        service, driver = None, None
+        '''Handle the "Detect" button's "clicked" signal.'''
 
+        @thread_callback
         def report_failure(message):
-            '''
-            Handle failures reported by the service backend.
-            '''
+            '''Handle failures reported by the service backend.'''
 
             self.__progressbar_detect_basics.hide()
             show_message(self.__dialog, _('Remote Configuration Failed'), message)
@@ -404,10 +422,9 @@ class CustomConfiguration(object):
             if service:
                 service.ManageLircDaemon('start')
 
+        @thread_callback
         def report_success(result, sender=None):
-            '''
-            Handle success reported by the service backend.
-            '''
+            '''Handle success reported by the service backend.'''
 
             self.__progressbar_detect_basics.hide()
 
@@ -423,33 +440,44 @@ class CustomConfiguration(object):
             if service:
                 service.ManageLircDaemon('start')
 
+        @thread_callback
         def report_progress(sender=None):
-            '''
-            Handle progress reported by the service backend.
-            '''
+            '''Handle progress reported by the service backend.'''
 
             self.__progressbar_detect_basics.pulse()
 
+        @thread_callback
         def request_action(message, details=None, sender=None):
-            '''
-            Forward actions requests from the service backend to the user.
-            '''
+            '''Forward actions requests from the service backend to the user.'''
 
             self.__progressbar_detect_basics.set_text(message)
             self.__progressbar_detect_basics.set_fraction(0)
 
             if details:
-                show_message(self.__dialog, message, details, type=gtk.MESSAGE_INFO)
+                # TODO: This dialog should probably have a cancel button.
+                response_buttons = (
+                    (gtk.RESPONSE_ACCEPT, _('_Start'), gtk.STOCK_EXECUTE),
+                )
+
+                show_message(self.__dialog, message, details,
+                             message_type=gtk.MESSAGE_INFO,
+                             buttons=response_buttons)
 
             driver.Proceed()
 
+        bus, service, driver = None, None, None
+
         try:
+            self.__stop_detection()
+
             bus     = backend.get_service_bus()
             service = backend.get_service(bus)
 
             driver = service.DetectParameters(self.__receiver.lirc_driver or '',
                                               self.__device or '')
+
             driver = bus.get_object(service.requested_bus_name, driver)
+            driver = dbus.Interface(driver, backend.ExternalToolDriver.INTERFACE_NAME)
 
             driver.connect_to_signal('RequestAction',  request_action)
             driver.connect_to_signal('ReportProgress', report_progress)
@@ -458,28 +486,44 @@ class CustomConfiguration(object):
 
             # TODO: Stop the key-listener, when we know that lircd is disabled.
             service.ManageLircDaemon('stop')
-            driver.Execute()
+
+            self.__detection_driver = driver
+            self.__detection_driver.Execute()
 
             self.__progressbar_detect_basics.show()
 
         except dbus.DBusException, ex:
-            report_failure(ex.message)
+            report_failure.__direct__(ex.message)
+            self.__stop_detection()
+
+    def __stop_detection(self):
+        '''Stop the basic properties detection driver.'''
+
+        try:
+            if self.__detection_driver:
+                self.__detection_driver.Release()
+                self.__detection_driver = None
+
+        except dbus.DBusException, ex:
+            logging.warning(ex)
 
     def _on_dialog_changed(self, widget = None):
-        '''
-        Handle major changes to the dialog.
-        '''
+        '''Handle major changes to the dialog.'''
 
         # Calculate completion state
 
         have_receiver = (None != self.__receiver)
         have_basics = bool(self.__treeview_basics.get_model())
         have_keys = bool(self.__treeview_keys.get_model())
+        is_learning = self.__learning_row is not None
 
         model_complete = (bool(self.vendor_name) and bool(self.product_name))
         basics_complete = (model_complete and have_basics)
         keys_complete = (basics_complete and have_keys)
 
+        selection = self.__treeview_keys.get_selection()
+        keys_model, keys_iter = selection.get_selected()
+
         # Update state of global widgets:
         self.__button_ok.set_sensitive(model_complete)
 
@@ -494,13 +538,15 @@ class CustomConfiguration(object):
         # Update state of keys-page widgets:
         self.__treeview_keys.set_sensitive(basics_complete and have_receiver)
         self.__hbuttonbox_keys.set_sensitive(basics_complete and have_receiver)
-        self.__button_keys_remove.set_sensitive(have_keys)
-        self.__button_keys_clear.set_sensitive(have_keys)
+
+        self.__button_keys_add.set_sensitive(not is_learning)
+        self.__button_keys_remove.set_sensitive(have_keys and not is_learning)
+        self.__button_keys_clear.set_sensitive(have_keys and not is_learning)
+        self.__button_keys_learn.set_sensitive(keys_iter is not None)
+        self.__button_keys_learn.set_active(is_learning)
 
     def select_key(self, key_name):
-        '''
-        Select the row of the specified key.
-        '''
+        '''Select the row of the specified key.'''
 
         keys_model = self.__treeview_keys.get_model()
         tree_iter = keys_model.find_iter(key_name)
@@ -520,14 +566,10 @@ class CustomConfiguration(object):
 
     # pylint: disable-msg=W0613
     def _on_button_keys_add_clicked(self, button):
-        '''
-        Handle the "Add" button's "clicked" signal.
-        '''
+        '''Handle the "Add" button's "clicked" signal.'''
 
         def find_next_key():
-            '''
-            Find the next unassigned key.
-            '''
+            '''Find the next unassigned key.'''
 
             # Figure out if one of the default keys is missing:
 
@@ -576,9 +618,7 @@ class CustomConfiguration(object):
         self.__start_learning(path)
 
     def _on_button_keys_remove_clicked(self, button):
-        '''
-        Handle the "Remove" button's "clicked" signal.
-        '''
+        '''Handle the "Remove" button's "clicked" signal.'''
 
         selection = self.__treeview_keys.get_selection()
         keys_model, tree_iter = selection.get_selected()
@@ -590,20 +630,28 @@ class CustomConfiguration(object):
         selection.select_path(path)
         self._on_dialog_changed()
 
+    def _on_button_keys_learn_toggled(self, button):
+        '''Handle the "Learn Key Code" button's "clicked" signal.'''
+
+        if self.__button_keys_learn.get_active():
+            selection = self.__treeview_keys.get_selection()
+            keys_model, keys_iter = selection.get_selected()
+            path = keys_model.get_path(keys_iter)
+            self.__start_learning(path)
+
+        else:
+            self.__stop_learning()
+
     def _on_button_keys_clear_clicked(self, button):
-        '''
-        Handle the "Clear" button's "clicked" signal.
-        '''
+        '''Handle the "Clear" button's "clicked" signal.'''
 
         self.__treeview_keys.get_model().clear()
         self._on_dialog_changed()
 
     def _on_close(self, dialog):
-        '''
-        Handle the dialog's "close" signal.
-        '''
+        '''Handle the dialog's "close" signal.'''
 
-        # TODO: Also release detection driver if needed.
+        self.__stop_detection()
         self.__stop_learning()
 
         # Prevent that pygtk destroys the dialog on ESC:
@@ -612,9 +660,7 @@ class CustomConfiguration(object):
         return True
 
     def _on_response(self, dialog, response):
-        '''
-        Handle the dialog's "response" signal.
-        '''
+        '''Handle the dialog's "response" signal.'''
 
         if gtk.RESPONSE_OK == response:
             try:
@@ -633,9 +679,7 @@ class CustomConfiguration(object):
             dialog.stop_emission('response')
 
     def run(self, receiver, device, remote):
-        '''
-        Show the dialog and return when the window is hidden.
-        '''
+        '''Show the dialog and return when the window is hidden.'''
 
         self.__apply_hardware(receiver, device, remote)
         self.__notebook.set_current_page(0)
